import { jsx as _jsx } from "react/jsx-runtime";
import get from "lodash/get";
import isEqual from "lodash/isEqual";
import merge from "lodash/merge";
import set from "lodash/set";
import { createContext, useEffect, useState } from "react";
import { v4 as uuidv4 } from "uuid";
import findFormControlError from "../utils/findFormControlError";
const defaultContextProps = {
    formControls: {},
    fileUploaderConfig: {},
    hasExpertMode: false,
    isExpertMode: false,
};
export const FormDataContext = createContext(defaultContextProps);
export const getControlByPath = (formControls, path) => {
    return get(formControls, path);
};
export const getEnhancedControls = (controls) => {
    return Object.entries(controls)
        .map(([name, control]) => ({ [name]: merge(control, { key: uuidv4() }) }))
        .reduce((prev, cur) => merge(prev, cur), {});
};
export const FormDataProvider = ({ children, controls, fileUploaderConfig, resolveUrl, postGenAiMessage, }) => {
    const [formControls, setFormControls] = useState(controls || defaultContextProps.formControls);
    const [hasExpertMode, setHasExpertMode] = useState(defaultContextProps.hasExpertMode);
    const [isExpertMode, setIsExpertMode] = useState(defaultContextProps.isExpertMode);
    const [toggleRerendered, setToggleRerendered] = useState(false);
    useEffect(() => {
        if (!isEqual(controls, formControls)) {
            setFormControls(controls);
        }
        // used for textarea convert when submit failed and we get the converted value from backend
        setToggleRerendered((prev) => !prev);
        //* enable expert mode when control with expert mode has error
        if (findFormControlError(controls, "expert")) {
            setIsExpertMode(true);
        }
    }, [controls]);
    const updateControl = (objectPathId, control) => {
        setFormControls((formControls) => {
            const existingErrors = get(formControls, `${objectPathId}.error`, null);
            const updatedControl = existingErrors
                ? { ...control, error: existingErrors } // keep existing errors (DEV-6411)
                : control;
            return merge({}, set(formControls, objectPathId, updatedControl));
        });
    };
    const addListItem = (objectPathId) => {
        setFormControls((formControls) => {
            const templateObjectPathId = objectPathId.replace(/items.\[\d+\]/g, "template");
            const currentControl = (get(formControls, objectPathId) ||
                get(formControls, templateObjectPathId));
            const template = currentControl.template;
            const newItem = getEnhancedControls(merge({}, template));
            return merge({}, set(formControls, `${objectPathId}.items`, [newItem].concat(currentControl.items || [])));
        });
    };
    const removeListItem = (objectPathId, index) => {
        setFormControls((formControls) => {
            const currentControl = get(formControls, objectPathId);
            const updatedItems = currentControl.items.filter((_, i) => i !== index);
            return merge({}, set(formControls, `${objectPathId}.items`, updatedItems));
        });
    };
    const moveListItemUp = (objectPathId, index) => {
        setFormControls((formControls) => {
            const currentControl = get(formControls, objectPathId);
            const items = currentControl.items;
            const item1 = items[index];
            items[index] = items[index - 1];
            items[index - 1] = item1;
            return merge({}, set(formControls, `${objectPathId}.items`, currentControl.items));
        });
    };
    const moveListItemDown = (objectPathId, index) => {
        setFormControls((formControls) => {
            const currentControl = get(formControls, objectPathId);
            const items = currentControl.items;
            const item1 = items[index];
            items[index] = items[index + 1];
            items[index + 1] = item1;
            return merge({}, set(formControls, `${objectPathId}.items`, currentControl.items));
        });
    };
    const getControlByObjectPath = (path) => {
        return getControlByPath(formControls, path);
    };
    const enableExpertMode = () => setHasExpertMode(true);
    const toggleExpertMode = () => setIsExpertMode((isExpertMode) => !isExpertMode);
    return (_jsx(FormDataContext.Provider, { value: {
            formControls,
            fileUploaderConfig,
            hasExpertMode,
            isExpertMode,
            enableExpertMode,
            toggleExpertMode,
            resolveUrl,
            updateControl,
            addListItem,
            moveListItemUp,
            moveListItemDown,
            removeListItem,
            getControlByObjectPath,
            toggleRerendered,
            postGenAiMessage,
        }, children: children }));
};
//# sourceMappingURL=FormData.js.map